/**
 * webgl_geometry_minecraft.html
 */

THREE.World = function ( seed, worldWidth, worldDepth ) {

	var worldHalfWidth = worldWidth / 2;
	var worldHalfDepth = worldDepth / 2;

	this.mesh = null;

	this.mt = new MersenneTwister();
	this.mt.setSeed( seed );

	this.generateHeight = function( width, height ) {

		var data = [], perlin = new ImprovedNoise(),
		size = width * height, quality = 2, z = this.mt.next() * 100;

		for ( var j = 0; j < 4; j ++ ) {

			if ( j == 0 ) for ( var i = 0; i < size; i ++ ) data[ i ] = 0;

			for ( var i = 0; i < size; i ++ ) {

				var x = i % width, y = ~~ ( i / width );
				data[ i ] += perlin.noise( x / quality, y / quality, z ) * quality;


			}

			quality *= 4

		}

		return data;

	}

	this.loadTexture = function( path ) {

		var image = new Image();
		image.onload = function () { texture.needsUpdate = true; };
		image.src = path;

		var texture  = new THREE.Texture( image, new THREE.UVMapping(), THREE.ClampToEdgeWrapping, THREE.ClampToEdgeWrapping, THREE.NearestFilter, THREE.LinearMipMapLinearFilter );

		return new THREE.MeshLambertMaterial( { map: texture, ambient: 0xbbbbbb } );

	}

	this.getY = function( x, z ) {

		return ~~( this.data[ x + z * worldWidth ] * 0.2 );

	}

	this.data = this.generateHeight( worldWidth, worldDepth );

	this.grass_dirt = this.loadTexture( '../three/grass_dirt.png' );
	this.grass = this.loadTexture( '../three/grass.png' );
	this.dirt = this.loadTexture( '../three/dirt.png' );

	this.materials = [

		this.grass_dirt, // right
		this.grass_dirt, // left
		this.grass, // top
		this.dirt, // bottom
		this.grass_dirt, // back
		this.grass_dirt  // front

	];

	var h, h2, px, nx, pz, nz, cubes = [];

	for ( var i = 0; i < 16; i++ ) {

		px = ( i & 8 ) == 8;
		nx = ( i & 4 ) == 4;
		pz = ( i & 2 ) == 2;
		nz = ( i & 1 ) == 1;
		cubes[ i ] = new THREE.CubeGeometry( 100, 100, 100, 1, 1, 1, this.materials, { px: px, nx: nx, py: true, ny: false, pz: pz, nz: nz } );

	}

	var geometry = new THREE.Geometry();

	for ( var z = 0; z < worldDepth; z ++ ) {

		for ( var x = 0; x < worldWidth; x ++ ) {

			px = nx = pz = nz = 0;

			h = this.getY( x, z );

			h2 = this.getY( x + 1, z );
			px = ( h2 != h && h2 != h + 1 ) || x == 0 ? 1 : 0;

			h2 = this.getY( x - 1, z );
			nx = ( h2 != h && h2 != h + 1 ) || x == worldWidth - 1 ? 1 : 0;

			h2 = this.getY( x, z + 1 );
			pz = ( h2 != h && h2 != h + 1 ) || z == worldDepth - 1 ? 1 : 0;

			h2 = this.getY( x, z - 1 );
			nz = ( h2 != h && h2 != h + 1 ) || z == 0 ? 1 : 0;

			this.mesh = new THREE.Mesh( cubes[ px * 8 + nx * 4 + pz * 2 + nz ] );

			this.mesh.position.x = x * 100 - worldHalfWidth * 100;
			this.mesh.position.y = h * 100;
			this.mesh.position.z = z * 100 - worldHalfDepth * 100;

			THREE.GeometryUtils.merge( geometry, this.mesh );

		}

	}

	this.mesh = new THREE.Mesh( geometry, new THREE.MeshFaceMaterial() );

};
